﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Infragistics.Windows;
using Infragistics.Windows.DataPresenter;



namespace SSI.Windows.InfragisticsExtensions
{
	public interface IXamDataPresenterFieldAdorner
	{
		Boolean IsAttached { get; set; }
		void Attach (DataPresenterBase presenter);
	}

	public class XamDataPresenterFieldAdorner : ContentControl, IXamDataPresenterFieldAdorner
	{
		#region Construction
		static XamDataPresenterFieldAdorner ()
		{
			DefaultStyleKeyProperty.OverrideMetadata(typeof(XamDataPresenterFieldAdorner), new FrameworkPropertyMetadata(typeof(XamDataPresenterFieldAdorner)));
		}

		public XamDataPresenterFieldAdorner ()
		{
			// Hide the control when the user clicks a Close button.
			this.CommandBindings.Add(new CommandBinding(ApplicationCommands.Close, this.Close, this.CanClose));

			// When the DataContext changes, that means this control
			// is being shown for a different row than before.  In that
			// case we have to clean out some previous settings associated
			// with the previously adorned cell.
			this.DataContextChanged += new DependencyPropertyChangedEventHandler(this.XamDataPresenterFieldAdorner_DataContextChanged);
		}
		#endregion

		#region Properties
		public DataPresenterBase Presenter { get; private set; }
		public Cell AdornedCell { get; private set; }

		public Boolean IsClosed
		{
			get
			{
				return (this.Visibility != Visibility.Visible);
			}
			private set
			{
				this.Visibility = value ? Visibility.Collapsed : Visibility.Visible;
				if (value && this.IsKeyboardFocusWithin)
					this.Presenter.Focus();
			}
		}
		#endregion

		#region Interface
		public void Close ()
		{
			this.IsClosed = true;
		}
		#endregion

		#region IXamDataPresenterFieldAdorner
		public Boolean IsAttached { get; set; }

		public void Attach (DataPresenterBase presenter)
		{
			this.Presenter = presenter ?? throw new ArgumentNullException("presenter");
			this.Presenter.EditModeStarted += (sender, e) =>
			{
				if (this.IsClosed)
					this.IsClosed = false;
			};

			this.IsAttached = true;
		}
		#endregion

		#region Event Handlers
		//protected override void OnPreviewKeyDown (KeyEventArgs e)
		//{
		//	RoutedCommand cmd = this.GetCommandForKey(e.Key, e.KeyboardDevice);
		//	if (cmd != null)
		//	{
		//		e.Handled = true;

		//		this.Presenter.ExecuteCommand(cmd);

		//		// Set focus to the grid because our TextBox might have focus now.
		//		// This is necessary because none of the grid cells are editable.
		//		this.Presenter.Focus();
		//	}

		//	base.OnPreviewKeyDown(e);
		//}

		protected override void OnPreviewMouseDown (MouseButtonEventArgs e)
		{
			// If the user clicks on a TextBox, give it focus and select all the text it contains.
			DependencyObject depobj = e.OriginalSource as DependencyObject;
			if (Utilities.GetAncestorFromType(depobj, typeof(TextBox), false, this) is TextBox txt && !txt.IsFocused)
			{
				txt.Focus();
				txt.SelectAll();
				e.Handled = true;
			}

			base.OnPreviewMouseDown(e);
		}

		private void XamDataPresenterFieldAdorner_DataContextChanged (Object sender, DependencyPropertyChangedEventArgs e)
		{
			if (this.IsClosed)
				this.IsClosed = false;

			this.AdornedCell = this.Presenter.ActiveCell;
		}
		#endregion

		#region Close Command
		private void CanClose (Object sender, CanExecuteRoutedEventArgs e)
		{
			e.CanExecute = true;
			e.Handled = true;
		}

		private void Close (Object sender, ExecutedRoutedEventArgs e)
		{
			this.IsClosed = true;
		}
		#endregion

		#region Private Helpers
		//private RoutedCommand GetCommandForKey (Key key, KeyboardDevice device)
		//{
		//	RoutedCommand cmd = null;

		//	if (key == Key.Tab)
		//	{
		//		// If the Shift key is pressed, give focus back to the active cell.
		//		// If the user only pressed Tab, move to the next cell.
		//		cmd = device.Modifiers == ModifierKeys.Shift ? DataPresenterCommands.StartEditMode : DataPresenterCommands.CellNextByTab;
		//	}
		//	else if (!this.IsTabStop)
		//	{
		//		// If this control is not a tab stop, it is still possible
		//		// that the user clicked on a child control to give it focus.
		//		// In that case, we must manually move focus to the correct cell.
		//		if (key == Key.Right)
		//		{
		//			cmd = DataPresenterCommands.CellRight;
		//		}
		//		else if (key == Key.Left)
		//		{
		//			cmd = DataPresenterCommands.CellLeft;
		//		}
		//		else if (key == Key.Escape)
		//		{
		//			// Move input focus back to the active cell.
		//			cmd = DataPresenterCommands.EndEditModeAndDiscardChanges;
		//		}
		//	}

		//	return (cmd);
		//}
		#endregion
	}
}