﻿using System;
using System.Collections;
using System.Globalization;
using System.Windows;
using System.Windows.Data;



namespace SSI.Windows.Simulation.Conjoint
{
	internal class NullVisibilityConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			if (value == null)
				return ("Collapsed");
			return ("Visible");
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class FunctionHelperTextConverter : IMultiValueConverter
	{
		public Object Convert (Object[] values, Type targetType, Object parameter, CultureInfo culture)
		{
			try
			{
				String text = values[0] as String;
				//var levellist = values[1] as IEnumerable<AttributeHelperEditorViewModel.AttributeHelperInfo.LevelInfoItem>;
				//if (String.IsNullOrEmpty(text) || levellist == null)
				//{
				//	Trace.WriteLine("highlighting text or level list is null");
				//	return (null);
				//}
				//var valuemappings = levellist.ToDictionary(k => k.Label, v => System.Convert.ToDouble(v.Value, SimulatorApp.AppSettings.CurrentCulture));
				//var result = FormulaHelpers.GetHelperHighlighting(FormulaHelpers.HelperTexts, text, FormulaHelpers.NoMatchText, valuemappings, SimulatorApp.AppSettings.CurrentCulture);
				//Trace.WriteLine(String.Format("highlighting <{0}>,<{1}>,<{2}>", result[0] ?? "null", result[1] ?? "null", result[2] ?? "null"));
				switch (parameter as String)
				{
					case ("Pre"):
						return (text);
					case ("High"):
						return (null);
					case ("Post"):
						return (null);
				}
				return (null);
			}
			catch
			{
				return (null);
			}
		}

		public Object[] ConvertBack (Object value, Type[] targetTypes, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
	}

	internal class FunctionHelperTextVisibilityConverter : IMultiValueConverter
	{
		public Object Convert (Object[] values, Type targetType, Object parameter, CultureInfo culture)
		{
			try
			{
				String text = values[0] as String;
				//var levellist = values[1] as IEnumerable<AttributeHelperEditorViewModel.AttributeHelperInfo.LevelInfoItem>;
				//if (text == null || levellist == null)
				//	return (Visibility.Collapsed);
				//var valuemappings = levellist.ToDictionary(k => k.Label, v => System.Convert.ToDouble(v.Value, SimulatorApp.AppSettings.CurrentCulture));
				//var result = FormulaHelpers.GetHelperHighlighting(FormulaHelpers.HelperTexts, text, FormulaHelpers.NoMatchText, valuemappings, SimulatorApp.AppSettings.CurrentCulture);
				//switch (parameter as String)
				//{
				//	case ("Pre"):
				//		return (result[0] != null ? Visibility.Visible : Visibility.Collapsed);
				//	case ("High"):
				//		return (result[1] != null ? Visibility.Visible : Visibility.Collapsed);
				//	case ("Post"):
				//		return (result[2] != null ? Visibility.Visible : Visibility.Collapsed);
				//}
				//Trace.WriteLine(String.Format("visibility <{0}>,<{1}>,<{2}>", result[0] ?? "null", result[1] ?? "null", result[2] ?? "null"));
				return (text.StartsWith("=") ? Visibility.Visible : Visibility.Collapsed);
			}
			catch
			{
				return (Visibility.Collapsed);
			}
		}

		public Object[] ConvertBack (Object value, Type[] targetTypes, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
	}

	internal class InverseNullVisibilityConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			if (value == null)
				return ("Visible");
			return ("Collapsed");
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class InverseBooleanVisibilityConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			if ((Boolean) value)
				return ("Collapsed");
			return ("Visible");
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class BooleanHiddenVisibilityConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			if ((Boolean) value)
				return ("Visible");
			return ("Hidden");
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class CountZeroVisibilityConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			if ((Int32) value == 0)
				return ("Visible");
			return ("Collapsed");
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class ToUpperCaseConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			return ((String) value).ToUpper();
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class InverseVisibilityConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			Visibility vis = (Visibility) value;
			if (vis != Visibility.Visible)
				return ("Visible");
			return ("Collapsed");
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class PriceAttributeIdxVisibilityConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			if ((Int32) value == -1)
				return ("Hidden");
			return ("Visible");
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class ScenarioRibbonLabelConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			if (value == null)
				return (null);
			return (String.Format(" {0} Settings ", (String) value)); //-The padding helps the QAT
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class EnumerableCountVisibilityConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			IList col = value as IList;
			if (col != null && col.Count > 0)
				return ("Visible");
			return ("Collapsed");
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class PercentageConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			Double pct = (Double) value;
			pct /= 100.0;
			return (pct.ToString("P"));
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class DebugConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			System.Diagnostics.Debugger.Break();
			return (value);
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			System.Diagnostics.Debugger.Break();
			return (value);
		}
		#endregion
	}

	internal class ProhibitedColorConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			if ((Boolean) value)
				return ("Red");
			return ("Transparent");
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class BooleanBoldConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			if ((Boolean) value)
				return ("Bold");
			return ("Normal");
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class InverseBooleanConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			return (!((Boolean) value));
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			return (!((Boolean) value));
		}
		#endregion
	}

	internal class StringNullOrEmptyVisibilityConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			if (String.IsNullOrEmpty((String) value))
				return ("Collapsed");
			return ("Visible");
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class AccessKeyRemovalConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			if (String.IsNullOrEmpty((String) value))
				return (String.Empty);
			return (((String) value).Replace("_", "__"));
		}

		public Object ConvertBack (Object value, Type targetType, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class MultiVisibilityConverter : IMultiValueConverter
	{
		#region IMultiValueConverter Members
		public Object Convert (Object[] values, Type targetType, Object parameter, CultureInfo culture)
		{
			// this will return visible if all the inputs return visible, otherwise collapsed
			foreach (var o in values)
			{
				var vis = (Visibility) o;
				if (vis != Visibility.Visible)
					return (Visibility.Collapsed);
			}

			return (Visibility.Visible);
		}

		public Object[] ConvertBack (Object value, Type[] targetTypes, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class MultiBooleanVisibilityConverter : IMultiValueConverter
	{
		#region IMultiValueConverter Members
		public Object Convert (Object[] values, Type targetType, Object parameter, CultureInfo culture)
		{
			String logic = (String) parameter;

			// these are the visibilities for 'OR' logic where any can be true to return visible
			Visibility conditionalvis = Visibility.Visible;
			Boolean conditionalvalue = true;
			Visibility defaultvis = Visibility.Collapsed;

			if (logic == "AND") // all must be true to return visible
			{
				conditionalvis = Visibility.Collapsed;
				conditionalvalue = false;
				defaultvis = Visibility.Visible;
			}

			foreach (var o in values)
			{
				if (((Boolean) o) == conditionalvalue)
					return (conditionalvis);
			}
			return (defaultvis);
		}

		public Object[] ConvertBack (Object value, Type[] targetTypes, Object parameter, CultureInfo culture)
		{
			throw (new NotImplementedException());
		}
		#endregion
	}

	internal class EnumBooleanConverter : IValueConverter
	{
		#region IValueConverter Members
		public Object Convert (Object value, Type targettype, Object parameter, CultureInfo culture)
		{
			var parameterstring = parameter as String;
			if (parameterstring == null)
				return (DependencyProperty.UnsetValue);

			if (Enum.IsDefined(value.GetType(), value) == false)
				return (DependencyProperty.UnsetValue);

			var paramvalue = Enum.Parse(value.GetType(), parameterstring);
			return (paramvalue.Equals(value));
		}

		public Object ConvertBack (Object value, Type targettype, Object parameter, CultureInfo culture)
		{
			var parameterstring = parameter as String;
			var valueasboolean = (Boolean) value;

			if (parameterstring == null || !valueasboolean)
			{
				try
				{
					return (Enum.Parse(targettype, "0"));
				}
				catch (Exception)
				{
					return (DependencyProperty.UnsetValue);
				}
			}

			return (Enum.Parse(targettype, parameterstring));
		}
		#endregion
	}
}